Package: dtplyr
Title: Data Table Back-End for 'dplyr'
Version: 1.3.3
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("cre", "aut")),
    person("Maximilian", "Girlich", role = "aut"),
    person("Mark", "Fairbanks", role = "aut"),
    person("Ryan", "Dickerson", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Provides a data.table backend for 'dplyr'. The goal of
    'dtplyr' is to allow you to write 'dplyr' code that is automatically
    translated to the equivalent, but usually much faster, data.table
    code.
License: MIT + file LICENSE
URL: https://dtplyr.tidyverse.org, https://github.com/tidyverse/dtplyr
BugReports: https://github.com/tidyverse/dtplyr/issues
Depends: R (>= 4.0)
Imports: cli (>= 3.4.0), data.table (>= 1.13.0), dplyr (>= 1.1.0),
        glue, lifecycle, rlang (>= 1.0.4), tibble, tidyselect (>=
        1.2.0), vctrs (>= 0.4.1)
Suggests: bench, covr, knitr, rmarkdown, testthat (>= 3.1.2), tidyr (>=
        1.1.0), waldo (>= 0.3.1)
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-02-10 17:24:49 UTC; hadleywickham
Author: Hadley Wickham [cre, aut],
  Maximilian Girlich [aut],
  Mark Fairbanks [aut],
  Ryan Dickerson [aut],
  Posit Software, PBC [cph, fnd]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-02-11 06:10:52 UTC
Built: R 4.6.0; ; 2026-04-28 22:44:28 UTC; unix
