# Generated by roxygen2: do not edit by hand

S3method(add_count,dtplyr_step)
S3method(anti_join,dtplyr_step)
S3method(arrange,dtplyr_step)
S3method(as.data.frame,dtplyr_step)
S3method(as.data.frame,foo)
S3method(as.data.table,dtplyr_step)
S3method(as_tibble,dtplyr_step)
S3method(auto_copy,dtplyr_step)
S3method(collect,dtplyr_step)
S3method(compute,dtplyr_step)
S3method(count,dtplyr_step)
S3method(dim,dtplyr_step)
S3method(dim,dtplyr_step_first)
S3method(distinct,dtplyr_step)
S3method(do,dtplyr_step)
S3method(dt_call,dtplyr_step)
S3method(dt_call,dtplyr_step_assign)
S3method(dt_call,dtplyr_step_first)
S3method(dt_call,dtplyr_step_join)
S3method(dt_call,dtplyr_step_modify)
S3method(dt_call,dtplyr_step_set)
S3method(dt_call,dtplyr_step_subset)
S3method(dt_has_computation,dtplyr_step)
S3method(dt_has_computation,dtplyr_step_first)
S3method(dt_has_computation,dtplyr_step_group)
S3method(dt_sources,dtplyr_step)
S3method(dt_sources,dtplyr_step_first)
S3method(dt_sources,dtplyr_step_join)
S3method(dt_sources,dtplyr_step_set)
S3method(dt_sources,dtplyr_step_subset)
S3method(full_join,dtplyr_step)
S3method(glimpse,dtplyr_step)
S3method(group_by,dtplyr_step)
S3method(group_map,dtplyr_step)
S3method(group_modify,dtplyr_step)
S3method(group_size,dtplyr_step)
S3method(group_vars,dtplyr_step)
S3method(groups,dtplyr_step)
S3method(head,dtplyr_step)
S3method(inner_join,dtplyr_step)
S3method(left_join,dtplyr_step)
S3method(mutate,dtplyr_step)
S3method(n_groups,dtplyr_step)
S3method(print,dtplyr_step)
S3method(pull,dtplyr_step)
S3method(relocate,dtplyr_step)
S3method(rename,dtplyr_step)
S3method(rename_with,dtplyr_step)
S3method(right_join,dtplyr_step)
S3method(same_src,dtplyr_step)
S3method(sample_frac,dtplyr_step)
S3method(sample_n,dtplyr_step)
S3method(select,dtplyr_step)
S3method(semi_join,dtplyr_step)
S3method(show_query,dtplyr_step)
S3method(slice,dtplyr_step)
S3method(slice_head,dtplyr_step)
S3method(slice_max,dtplyr_step)
S3method(slice_min,dtplyr_step)
S3method(slice_sample,dtplyr_step)
S3method(slice_tail,dtplyr_step)
S3method(summarise,dtplyr_step)
S3method(tail,dtplyr_step)
S3method(tally,dtplyr_step)
S3method(tbl_vars,dtplyr_step)
S3method(tbl_vars,foo)
S3method(tidyselect_data_has_predicates,dtplyr_step)
S3method(tidyselect_data_proxy,dtplyr_step)
S3method(transmute,dtplyr_step)
S3method(ungroup,dtplyr_step)
S3method(union_all,dtplyr_step)
S3method(unique,dtplyr_step)
export(.datatable.aware)
export(lazy_dt)
import(rlang)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,is.data.table)
importFrom(dplyr,add_count)
importFrom(dplyr,anti_join)
importFrom(dplyr,arrange)
importFrom(dplyr,auto_copy)
importFrom(dplyr,collect)
importFrom(dplyr,compute)
importFrom(dplyr,count)
importFrom(dplyr,distinct)
importFrom(dplyr,do)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,glimpse)
importFrom(dplyr,group_by)
importFrom(dplyr,group_map)
importFrom(dplyr,group_modify)
importFrom(dplyr,group_size)
importFrom(dplyr,group_vars)
importFrom(dplyr,groups)
importFrom(dplyr,inner_join)
importFrom(dplyr,intersect)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n_groups)
importFrom(dplyr,pull)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,right_join)
importFrom(dplyr,same_src)
importFrom(dplyr,sample_frac)
importFrom(dplyr,sample_n)
importFrom(dplyr,select)
importFrom(dplyr,semi_join)
importFrom(dplyr,setdiff)
importFrom(dplyr,show_query)
importFrom(dplyr,slice)
importFrom(dplyr,slice_head)
importFrom(dplyr,slice_max)
importFrom(dplyr,slice_min)
importFrom(dplyr,slice_sample)
importFrom(dplyr,slice_tail)
importFrom(dplyr,summarise)
importFrom(dplyr,tally)
importFrom(dplyr,tbl_vars)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(dplyr,union)
importFrom(dplyr,union_all)
importFrom(glue,glue)
importFrom(lifecycle,deprecated)
importFrom(tibble,as_tibble)
importFrom(tidyselect,everything)
importFrom(tidyselect,tidyselect_data_has_predicates)
importFrom(tidyselect,tidyselect_data_proxy)
importFrom(utils,head)
importFrom(utils,tail)
